// wsClientDlg.cpp : Implementation of CwsClientDlg
#include "stdafx.h"
#include "wsClientDlg.h"

/////////////////////////////////////////////////////////////////////////////
// CwsClientDlg

// Attention: the number of any items in array 
// should equivalent with enum enumWsStates value for item (=ws<Any>)
// see 'Show current state' in CwsClientDlg::OnStateChanged()
STATE CwsClientDlg::m_states[] =
{
	wsClosed,					_T("Closed"), 
    wsSetuping,					_T("Setuping"), 
    wsListening,				_T("Listening"), 
    wsResolvingHost,			_T("ResolvingHost"), 
    wsHostResolved,				_T("HostResolved"), 
    wsConnectionPending,		_T("ConnectionPending"), 
    wsConnected,				_T("Connected"), 
    wsPreparationSending,		_T("PreparationSending"), 
    wsSending,					_T("Sending"), 
    wsWaitingResponse,			_T("WaitingResponse"), 
    wsUnexpectedDisconnection,	_T("UnexpectedDisconnection"), 
    wsClosing,					_T("Closing"), 
    wsError,					_T("Error"),
    wsWaitingRecoveryConnect,	_T("WaitingRecoveryConnect"),
};

VOID __stdcall CwsClientDlg::OnStateChanged()
{
	if (!m_bProgress)
	{
		m_bProgress = TRUE;
		FrmRefresh();
	}

	// Get current state
	enum enumWsStates wsCurState = m_spWsDmur->State();
	BOOL bSendEnabled = FALSE;
	BOOL bShowStatus = TRUE;
	
	switch (wsCurState)
	{
	case wsError:
		{
			long nWsErr = 0L;
			if (m_spWsDmur->GetErrorInfo(&nWsErr, &m_bstrWsErrMsg.m_str) == VARIANT_TRUE)
			{
				m_nWsErr = (ULONG)nWsErr;
				AddCodeToErrList(m_nWsErr, FALSE);
				::PostMessage(m_hWnd, UM_ERROR, (WPARAM)1, 0);
			}
		}
	case wsClosed:
	case wsClosing:
		bSendEnabled = TRUE;	
		break;
	case wsListening:
	case wsSetuping:
	case wsResolvingHost:
	case wsHostResolved:
	case wsConnectionPending:
	case wsConnected:
	case wsPreparationSending:
	case wsSending:
		break;
	case wsWaitingResponse:
		if (m_bInRecoveryConnect)
		{
			// The end RecoveryConnect state (Successful connect)
            AfterRecoveryConnect();
		}
		break;
	case wsUnexpectedDisconnection:
		//
		// The begine RecoveryConnect state
        BeforeRecoveryConnect();
		break;
	case wsWaitingRecoveryConnect:
		//
		// Show RecoveryConnect count
		//BeforeRecoveryConnect()
		m_nCntRecoveryConnect = m_nCntRecoveryConnect + (ULONG)1L;
		_stprintf(m_szTmpBuff, _T("wsWaitingRecoveryConnect (%lu / %lu)"),
			m_nCntRecoveryConnect, MAXRECOVERYCONNECT);
		SetDlgItemText(IDC_STATE, m_szTmpBuff);
		bShowStatus = FALSE;
		break;
	default:
		SetDlgItemText(IDC_STATE, _T("(Unknown)"));
		bSendEnabled = TRUE;
		bShowStatus = FALSE;
		break;
	}
	if (bShowStatus)
	{
		// Show current state
		SetDlgItemText(IDC_STATE, m_states[(int)wsCurState].lpszState);
	}
	if (bSendEnabled)
	{
		OnReceiveComplete();
	}
}

VOID __stdcall CwsClientDlg::OnDataArrival(LONG bytesReceived, LONG lenHeader, LONG UpLoadCount)
{
	OnDataReceived(bytesReceived, lenHeader, UpLoadCount);
}

VOID CwsClientDlg::SendOperation()
{

#if !USE_INRANET
	DWORD dwRetVal = ::InternetAttemptConnect(0);
	if (dwRetVal != ERROR_SUCCESS)
	{
		DWORD dwCodeErr = ::GetLastError();
		if (dwCodeErr != 0)
		{
			LPVOID lpMsgBuf;
			::FormatMessage( 
				FORMAT_MESSAGE_ALLOCATE_BUFFER | 
				FORMAT_MESSAGE_FROM_SYSTEM | 
				FORMAT_MESSAGE_IGNORE_INSERTS,
				NULL,
				dwCodeErr,
				0, // Default language
				(LPTSTR) &lpMsgBuf,
				0,
				NULL);
			if (lpMsgBuf != NULL)
			{
				MessageBox((LPCTSTR)lpMsgBuf, 
					_T("CwsClientDlg::SendOperation()"), MB_OK | MB_ICONINFORMATION );
			}
			::LocalFree(lpMsgBuf);
		}
		return 0;
	}
#endif
	// Get message
	CComBSTR bstrData;
	GetDlgItemText(IDC_MESSAGE, bstrData.m_str);
	if (bstrData.Length())
	{
		////////////////////////////////
		// Reset current properties
		//
		CComBSTR bstrProp;
		
		// Remote host
		GetDlgItemText(IDC_REMOTEHOST, bstrProp.m_str);
		if (bstrProp.Length() == 0)
		{
			// Empty property
			::MessageBeep((DWORD)(-1));
			return ;
		}
		m_spWsDmur->PutRemoteHost((BSTR)bstrProp);

		// File name
		GetDlgItemText(IDC_FILENAME, bstrProp.m_str);
		m_spWsDmur->PutFileName((BSTR)bstrProp);
		
		// RemotePort
		BOOL bTrans = FALSE;
		UINT nRemotePort = GetDlgItemInt(IDC_REMOTEPORT, &bTrans, FALSE);
		if (!bTrans || nRemotePort == 0)
		{
			// Empty property
			::MessageBeep((DWORD)(-1));
			return ;
		}
		m_spWsDmur->PutRemotePort((long)nRemotePort);

		HWND hWndSend = GetDlgItem(IDC_SEND);
		::SetFocus(hWndSend);
		::EnableWindow(hWndSend, FALSE);

		/////////////////////////////////
		// Clear all output fields
		//
		LPCTSTR lpszEmpty = _T("");
		SetDlgItemText(IDC_RECEIVED, lpszEmpty);
		SetDlgItemText(IDC_STATE, lpszEmpty);
		SetDlgItemText(IDC_BYTESRECIEVED, lpszEmpty);
		SetDlgItemText(IDC_ERRORS, lpszEmpty);
		SetDlgItemInt(IDC_BYTESSEND, 0, FALSE);
		SetDlgItemInt(IDC_BYTESREMAINING, 0, FALSE);
		SetDlgItemInt(IDC_UPLOADCOUNT, 0, FALSE);
		
		/////////////////////////////////////////
		// Set initial values for all parameters
		//
		m_nContentLength
			= m_nTotalLengthHeaders
			= m_nCurLen
			= m_nLoadLen
			= m_nSC
			= m_nWsErr
			= m_nUpLoadCount
			= m_nCntRecoveryConnect
			= m_nInitialFileLength
			= 0;
		m_bInRecoveryConnect = FALSE;
		GetDlgItemText(IDC_FILENAME, bstrProp.m_str);
		m_bIsFile = (bstrProp.Length() > 0);
		
		m_bProgress = TRUE;
		FrmRefresh();

		/////////////////////////////////
		// Disabled progress bar
		//
		::ShowWindow(GetDlgItem(IDC_PROGCTRL1), SW_HIDE);
		
		////////////////////////////////
		// Try send message
		//
		VARIANT_BOOL bIsContinue = (VARIANT_BOOL)(IsDlgButtonChecked(IDC_CHKCONTINUEDL) == BST_CHECKED);
		m_bContinueDownload = (bIsContinue != VARIANT_FALSE);
		m_spWsDmur->SendData((BSTR)bstrData, bIsContinue);
		
		::EnableWindow(hWndSend, TRUE);
		::SetFocus(hWndSend);
	}
	else
	{
		// Empty send message
		::MessageBeep((DWORD)(-1));
	}
}

VOID CwsClientDlg::StopOperation()
{
	if (!m_bProgress)
		return ;

	// Clear MessageBeep
	BOOL bUseMessageBeep = m_bUseMessageBeep;
	m_bUseMessageBeep = FALSE;

	m_spWsDmur->Stop();
	
	if (m_bInRecoveryConnect)
	{
		// Clear RecoveryConnect state
		AfterRecoveryConnect();
	}
	
	SetDownLoadContinue(); 

	m_nLoadLen
		= m_nInitialFileLength
		= 0;

	// Clear Progress state
	m_bProgress = FALSE;
	FrmRefresh();
	
	// Restore MessageBeep
	m_bUseMessageBeep = bUseMessageBeep;
}

VOID CwsClientDlg::OnDataReceived(const LONG& bytesReceived, 
	const LONG& lenHeader, const LONG& UpLoadCount)
{
	BOOL bIsShowenData = FALSE;

	// Check Http status code
	BOOL bIsError = FALSE;
	ULONG nCurCS = (ULONG)m_spWsDmur->GetStatusCode(); 
	if (nCurCS > (ULONG)0L)
	{
		if (m_nSC != nCurCS)
		{
			m_nSC = nCurCS;
			if (m_nSC >= (ULONG)300L)
			{
				// On error show all received data
				USES_CONVERSION;
   				CComBSTR bstrData;
				CComBSTR bstrBuff;
				GetDlgItemText(IDC_RECEIVED, bstrBuff.m_str);
				bstrData = m_spWsDmur->GetData();
				bstrBuff.Append(bstrData.m_str, (int)bytesReceived);
				SetDlgItemText(IDC_RECEIVED, W2CT(bstrBuff.m_str));
				bIsShowenData = TRUE;

				AddCodeToErrList(m_nSC);
				::PostMessage(m_hWnd, UM_ERROR, (WPARAM)2, 0);
				bIsError = TRUE;
			}
		}
	}

	if (!bIsShowenData)
	{
		// Get & Show recieved string
		if (m_bIsFile)
		{
			// if exist file show header only
			if (lenHeader > 0L)
			{
				USES_CONVERSION;
   				CComBSTR bstrData;
				CComBSTR bstrBuff;
				GetDlgItemText(IDC_RECEIVED, bstrBuff.m_str);
				bstrData = m_spWsDmur->GetData();
				bstrBuff.Append(bstrData.m_str, (int)lenHeader);
				SetDlgItemText(IDC_RECEIVED, W2CT(bstrBuff.m_str));
				bIsShowenData = TRUE;
			}
		}
		else
		{
			// Show all received data otherwise
			USES_CONVERSION;
   			CComBSTR bstrData;
			CComBSTR bstrBuff;
			GetDlgItemText(IDC_RECEIVED, bstrBuff.m_str);
			bstrData = m_spWsDmur->GetData();
			bstrBuff.Append(bstrData.m_str, (int)bytesReceived);
			SetDlgItemText(IDC_RECEIVED, W2CT(bstrBuff.m_str));
			bIsShowenData = TRUE;
		}
	}

	// Get total bytes recieved
	// (TotalBytesRecieved includes all headers!)
	ULONG lnTotalBytes = (ULONG)m_spWsDmur->TotalBytesRecieved();
	_TCHAR szBytesRecieved[2*_MAX_PATH];
	int nMaxBytesRecieved = 2*_MAX_PATH;

	if (!bIsError)
	{
        // Check new RecoveryConnection
		if (m_nUpLoadCount != (ULONG)UpLoadCount)
		{
            // Save current download length & Clear ContentLength
			m_nLoadLen = m_nCurLen;
			m_nUpLoadCount = (ULONG)UpLoadCount;
			SetDlgItemInt(IDC_UPLOADCOUNT, (UINT)UpLoadCount, FALSE);
			m_nContentLength 
				= m_nInitialFileLength
				= (ULONG)0L;
			
		}
		
		// Try get ContentLength
		if (m_nContentLength == (ULONG)0L)
		{
			// Only first recieved block
			// contains correct ContentLength for all datas
			LONG lnLenCont = (LONG)m_spWsDmur->ContentLength();
			if (lnLenCont > 0L)
			{
                m_nContentLength = (ULONG)lnLenCont;
			}

			// Try get InitialFileLength
			if (m_bContinueDownload)
			{
				LONG lnInitialLength = (LONG)m_spWsDmur->GetInitialFileLength();
				if (lnInitialLength > 0L)
				{
					m_nInitialFileLength = (ULONG)lnInitialLength;
				}
			}
		}

		// Is header now?
		if (lenHeader > 0L)
		{
			m_nTotalLengthHeaders = m_nTotalLengthHeaders + (ULONG)lenHeader;
		}

		// Is ContentLength now?
		if (m_nContentLength > (ULONG)0L)
		{
			// Calculate procent recieved ContentLength data
			m_nCurLen = m_nInitialFileLength + lnTotalBytes - m_nTotalLengthHeaders;
			ULONG lDataLen = m_nInitialFileLength + m_nLoadLen + m_nContentLength;
            double dDataLen = (double)lDataLen;
            double nLen = (double)m_nCurLen;
            double nProc = (nLen * 100) / dDataLen;
			if (nProc > (double)100.0)
			{
				nProc = (double)100.0;
			}
			_stprintf(szBytesRecieved, _T("%lu / %lu : %5.2f%%"),
				m_nCurLen, lDataLen, nProc);
			SetDlgItemText(IDC_BYTESRECIEVED, szBytesRecieved);
			
			// Show ProgressBar
			HWND hWndCtrl = GetDlgItem(IDC_PROGCTRL1);
			if (hWndCtrl != NULL)
			{
				if (!::IsWindowVisible(hWndCtrl))
				{
					m_spProgressBar->PutMax((float)100.0);
					::ShowWindow(hWndCtrl, SW_SHOW);
				}
				if (nProc > (double)0.0)
					m_spProgressBar->PutValue((float)nProc);
				else
					m_spProgressBar->PutValue((float)0.0001);
			}
		}
		else
		{
			// Only TotalBytes recieved
			_stprintf(szBytesRecieved, _T("%lu"), lnTotalBytes);
			SetDlgItemText(IDC_BYTESRECIEVED, szBytesRecieved);
		}
	}
	else
	{
		// Show TotalBytes recieved (for empty prev BytesRecieved only)
		GetDlgItemText(IDC_BYTESRECIEVED, szBytesRecieved, nMaxBytesRecieved);
		if (_tcslen(szBytesRecieved) == 0)
		{
			_stprintf(szBytesRecieved, _T("%lu"), lnTotalBytes);
			SetDlgItemText(IDC_BYTESRECIEVED, szBytesRecieved);
		}
	}
}

VOID CwsClientDlg::AddCodeToErrList(ULONG nError, BOOL bHttpErr)
{
	if (nError > (ULONG)0L)
	{
		USES_CONVERSION;
		_TCHAR szCode[10];
		CComBSTR bstrProp;
		GetDlgItemText(IDC_ERRORS, bstrProp.m_str);
		if (bstrProp.Length() > 0)
		{
			bstrProp += _T(", ");
		}
		if (bHttpErr)
		{
			bstrProp += _T("Http-code: ");
		}
		_stprintf(szCode, _T("%lu"), nError);
		bstrProp += szCode;
		SetDlgItemText(IDC_ERRORS, W2CT(bstrProp.m_str));
		int nLen = (int)bstrProp.Length(); 
		SendDlgItemMessage(IDC_ERRORS, EM_SETSEL, (WPARAM)nLen, (LPARAM)nLen);
	}
}

VOID CwsClientDlg::SetDownLoadContinue()
{
	if (IsDlgButtonChecked(IDC_CHKCONTINUEDL) != BST_CHECKED)
	{
		// Set Download continue
		CheckDlgButton(IDC_CHKCONTINUEDL, BST_CHECKED);
		m_bContinueDownload = TRUE;
	}
}

VOID CwsClientDlg::OnReceiveComplete()
{
	// Check RecoveryConnect count
	if (m_bInRecoveryConnect)
	{
		if (m_nCntRecoveryConnect >= (ULONG)m_spWsDmur->GetMaxRecoveryConnect())
		{
			// The end RecoveryConnect state (on error)
            AfterRecoveryConnect();
			SetDownLoadContinue();
            m_nLoadLen = (ULONG)0L;
			m_bProgress = FALSE;
			FrmRefresh();
			if (m_bUseMessageBeep)
			{
				::MessageBeep((DWORD)(-1));
			}
		}
	}
	else
	{
		m_bProgress = FALSE;
		FrmRefresh();
		if (m_bUseMessageBeep)
		{
			::MessageBeep((DWORD)(-1));
		}
	}
}

